@@ -9,4 +9,7 @@
 #undef GLADE_LOCALE_DIR
 #undef GLADE_DATA_DIR
 #undef USE_GNOME
+#undef USE_GNOME_DB
+#undef ENABLE_BONOBO
 #undef PALETTE_TYPE
+#undef USING_OAF
