@@ -278,6 +278,7 @@ editor_show_grid_settings_dialog (void)
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title (GTK_WINDOW (window), _("Grid Options"));
   gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
+  gtk_window_set_wmclass (GTK_WINDOW (window), "grid_options", "Glade");
 
   vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), vbox);
@@ -440,6 +441,7 @@ editor_show_snap_settings_dialog (void)
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title (GTK_WINDOW (window), _("Snap Options"));
   gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
+  gtk_window_set_wmclass (GTK_WINDOW (window), "snap_options", "Glade");
 
   vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), vbox);
@@ -1097,6 +1099,19 @@ editor_on_button_press (GtkWidget * signal_widget,
 	  gtk_signal_emit_stop_by_name (GTK_OBJECT (signal_widget),
 					"button_press_event");
 	}
+      else
+        {
+         static gboolean already_shown = FALSE;
+         if (!already_shown)
+           {
+             glade_util_show_message_box (_("You can't add a widget at the selected position.\n"
+					    "\n"
+					    "Tip: GTK+ uses containers to lay out widgets.\n"
+					    "Try deleting the existing widget and using\n"
+					    "a box or table container instead.\n"));
+             already_shown = TRUE;
+           }
+        }
     }
   else if (event->button == 3)
     {
@@ -2591,9 +2606,15 @@ clear_child_windows (GdkWindow * window, gint x, gint y, gint w, gint h)
   GList *children;
   GdkWindow *child_window;
   gint win_x, win_y, win_w, win_h;
+#ifndef _WIN32
   XWindowAttributes xwa;
+#endif
   GdkRectangle area, child, intersection;
 
+#ifdef _WIN32
+  return;
+#endif
+
   area.x = x;
   area.y = y;
   area.width = w;
@@ -2616,6 +2637,7 @@ clear_child_windows (GdkWindow * window, gint x, gint y, gint w, gint h)
 	  /* We need to make sure this is not an InputOnly window, or we get
 	     a BadMatch. CList uses InputOnly windows - for resizing columns.
 	     FIXME: This may not be very efficient. */
+#ifndef _WIN32
 	  XGetWindowAttributes (GDK_DISPLAY (),
 				GDK_WINDOW_XWINDOW (child_window),
 				&xwa);
@@ -2631,6 +2653,7 @@ clear_child_windows (GdkWindow * window, gint x, gint y, gint w, gint h)
 				   intersection.x, intersection.y,
 				   intersection.width, intersection.height);
 	    }
+#endif
 	}
       children = children->next;
     }
