@@ -40,7 +40,7 @@ gb_widget_write_source (GtkWidget * widget,
   GtkWidget *parent;
   GbWidget *gbwidget;
   GbWidgetData *widget_data;
-  gchar *class_name, *child_name;
+  gchar *class_id, *child_name;
   gint source_len;
 
   /* This is a temporary(?) kludge so that the code for GtkDialogs is OK.
@@ -62,14 +62,14 @@ gb_widget_write_source (GtkWidget * widget,
 
   parent = data->parent;
 
-  class_name = gb_widget_get_type_name (widget);
+  class_id = gb_widget_get_class_id (widget);
   data->write_children = TRUE;
 
   widget_data = gtk_object_get_data (GTK_OBJECT (widget), GB_WIDGET_DATA_KEY);
   /* If this isn't a gbwidget, skip it. */
   if (widget_data)
     {
-      gbwidget = gb_widget_lookup_class (class_name);
+      gbwidget = gb_widget_lookup_class (class_id);
       g_return_if_fail (gbwidget != NULL);
 
       /* For custom widgets, we don't have a default widget to compare with,
@@ -80,24 +80,24 @@ gb_widget_write_source (GtkWidget * widget,
 	}
       else
 	{
-	  data->standard_widget = (GtkWidget *) g_hash_table_lookup (data->standard_widgets, class_name);
+	  data->standard_widget = (GtkWidget *) g_hash_table_lookup (data->standard_widgets, class_id);
 	  if (data->standard_widget == NULL)
 	    {
 #ifdef USE_GNOME
 	      /* FIXME: GnomeLibs 1.0.1 workaround - gtk_object_newv doesn't
 		 return a valid GnomeAppBar or GnomeDateEdit, so we create it
 		 ourself. */
-	      if (!strcmp (class_name, "GnomeAppBar"))
+	      if (!strcmp (class_id, "GnomeAppBar"))
 		data->standard_widget = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
-	      else if (!strcmp (class_name, "GnomeDateEdit"))
+	      else if (!strcmp (class_id, "GnomeDateEdit"))
 		data->standard_widget = gnome_date_edit_new ((time_t) 0, TRUE,
 							     TRUE);
 	      else
-		data->standard_widget = GTK_WIDGET (gtk_object_newv (gtk_type_from_name (class_name), 0, NULL));
+		data->standard_widget = GTK_WIDGET (gtk_object_newv (gtk_type_from_name (class_id), 0, NULL));
 #else
-	      data->standard_widget = GTK_WIDGET (gtk_object_newv (gtk_type_from_name (class_name), 0, NULL));
+	      data->standard_widget = GTK_WIDGET (gtk_object_newv (gtk_type_from_name (class_id), 0, NULL));
 #endif
-	      g_hash_table_insert (data->standard_widgets, class_name,
+	      g_hash_table_insert (data->standard_widgets, class_id,
 				   data->standard_widget);
 	    }
 	}
@@ -113,7 +113,7 @@ gb_widget_write_source (GtkWidget * widget,
       if (gbwidget->gb_widget_write_source)
 	(gbwidget->gb_widget_write_source) (widget, data);
       else
-	source_add (data, "  /* Skipping %s: unimplemented. */\n", class_name);
+	source_add (data, "  /* Skipping %s: unimplemented. */\n", class_id);
 
       /* Make sure there is a blank line after each widget, for readability. */
       source_len = data->source_buffers[GLADE_SOURCE]->len;
@@ -499,8 +499,12 @@ gb_widget_write_signals_source (GtkWidget * widget,
 						signal->data, handler);
 
       /* We don't need to output code for standard GTK functions. */
-      if (!strcmp (handler, "gtk_widget_hide")
-	  || !strcmp (handler, "gtk_widget_destroy"))
+      if (!strcmp (handler, "gtk_widget_show")
+	  || !strcmp (handler, "gtk_widget_hide")
+	  || !strcmp (handler, "gtk_widget_destroy")
+	  || !strcmp (handler, "gtk_true")
+	  || !strcmp (handler, "gtk_false")
+	  || !strcmp (handler, "gtk_main_quit"))
 	skip_handler_code = TRUE;
 
 
