@@ -102,7 +102,9 @@ struct _GbWidgetData
   guint		 cxx_use_heap : 1;
   guint		 cxx_separate_class : 1;
   guint		 cxx_visibility : 2; 
-  		/* 0=private 1=protected 2=public */
+
+  /* 0=private 1=protected 2=public */
+  GbWidget      *gbwidget;
 };
 
 typedef struct _GbSignal GbSignal;
@@ -354,6 +356,7 @@ struct _GbWidgetDestroyData
 struct _GbWidget
 {
   gchar		**pixmap_struct;
+  gchar          *class_id;
   GdkPixmap	 *gdkpixmap;
   GdkBitmap	 *mask;
   const gchar	 *tooltip;
@@ -440,24 +443,24 @@ struct _GbWidget
 void	    gb_widgets_init();
 
 /* This registers a GbWidget, so it can be looked up by class name. */
-void	    gb_widget_register_gbwidget (const gchar	       *class_name,
+void	    gb_widget_register_gbwidget (const gchar	       *class_id,
 					 GbWidget	       *gbwidget);
 /* Thes return the GbWidget struct corresponding to the given class name or
    widget. */
-GbWidget *  gb_widget_lookup_class	(const gchar	       *class_name);
+GbWidget *  gb_widget_lookup_class	(const gchar	       *class_id);
 GbWidget *  gb_widget_lookup		(GtkWidget	       *widget);
 
-/* This returns the name of the class corresponding to the given widget.
+/* This returns the class id corresponding to the given widget.
    This should be used instead of gtk_type_name so custom widgets work. */
-gchar*	    gb_widget_get_type_name	(GtkWidget	       *widget);
+gchar*	    gb_widget_get_class_id	(GtkWidget	       *widget);
 
 /* This initializes the GbWidget struct, setting all function pointers to NULL.
  */
 void	    gb_widget_init_struct	(GbWidget	       *gbwidget);
 
-GtkWidget*  gb_widget_new		(const gchar	       *class_name,
+GtkWidget*  gb_widget_new		(const gchar	       *class_id,
 					 GtkWidget	       *parent);
-GtkWidget*  gb_widget_new_full		(const gchar	       *class_name,
+GtkWidget*  gb_widget_new_full		(const gchar	       *class_id,
 					 gboolean	        create_default_name,
 					 GtkWidget	       *parent,
 					 GtkWidget	       *current_child,
@@ -466,12 +469,10 @@ GtkWidget*  gb_widget_new_full		(const gchar	       *class_name,
 					 GbWidgetNewCallback	callback,
 					 GbWidgetAction		action,
 					 GbWidgetSetArgData    *loading_data);
-GbWidgetData* gb_widget_new_widget_data (void);
+GbWidgetData* gb_widget_new_widget_data (GbWidget              *gbwidget);
 void	    gb_widget_create_from	(GtkWidget	       *widget,
 					 const gchar	       *name);
 gboolean    gb_widget_can_finish_new	(GbWidgetNewData       *data);
-void        gb_widget_real_initialize   (GtkWidget	       *widget,
-					 GbWidgetData	       *wdata);
 void        gb_widget_initialize	(GtkWidget	       *widget,
 					 GbWidgetNewData       *data);
 void	    gb_widget_free_new_data	(GbWidgetNewData       *data);
