@@ -18,11 +18,12 @@
 
 #include <string.h>
 
-#include <gtk/gtk.h>
+#include <gtk/gtkctree.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkspinbutton.h>
 #include "../gb.h"
 
-#ifdef GLD_HAVE_GTK_1_1
-
 /* Include the 21x21 icon pixmap for this widget, to be used in the palette */
 #include "../graphics/ctree.xpm"
 
@@ -38,25 +39,23 @@ static gchar *CTreeTitle = "CTree:title";
 static gchar *Mode = "CTree|GtkCList::selection_mode";
 static gchar *Titles = "CTree|GtkCList::show_titles";
 static gchar *Shadow = "CTree|GtkCList::shadow_type";
-static gchar *HPolicy = "CTree|GtkCList::hscrollbar_policy";
-static gchar *VPolicy = "CTree|GtkCList::vscrollbar_policy";
 
 /* This is only used for loading & saving - it isn't displayed in the property
    editor. */
 static gchar *Cols = "GtkCTree::columns";
 static gchar *ColumnWidths = "GtkCTree::column_widths";
 
-static gchar *GbModeChoices[] =
+static const gchar *GbModeChoices[] =
 {"Single", "Browse", "Multiple",
  "Extended", NULL};
-static gint GbModeValues[] =
+static const gint GbModeValues[] =
 {
   GTK_SELECTION_SINGLE,
   GTK_SELECTION_BROWSE,
   GTK_SELECTION_MULTIPLE,
   GTK_SELECTION_EXTENDED
 };
-static gchar *GbModeSymbols[] =
+static const gchar *GbModeSymbols[] =
 {
   "GTK_SELECTION_SINGLE",
   "GTK_SELECTION_BROWSE",
@@ -64,10 +63,10 @@ static gchar *GbModeSymbols[] =
   "GTK_SELECTION_EXTENDED"
 };
 
-static gchar *GbShadowChoices[] =
+static const gchar *GbShadowChoices[] =
 {"None", "In", "Out",
  "Etched In", "Etched Out", NULL};
-static gint GbShadowValues[] =
+static const gint GbShadowValues[] =
 {
   GTK_SHADOW_NONE,
   GTK_SHADOW_IN,
@@ -75,7 +74,7 @@ static gint GbShadowValues[] =
   GTK_SHADOW_ETCHED_IN,
   GTK_SHADOW_ETCHED_OUT
 };
-static gchar *GbShadowSymbols[] =
+static const gchar *GbShadowSymbols[] =
 {
   "GTK_SHADOW_NONE",
   "GTK_SHADOW_IN",
@@ -84,28 +83,15 @@ static gchar *GbShadowSymbols[] =
   "GTK_SHADOW_ETCHED_OUT"
 };
 
-static gchar *GbPolicyChoices[] =
-{"Always", "Automatic", NULL};
-static gint GbPolicyValues[] =
-{
-  GTK_POLICY_ALWAYS,
-  GTK_POLICY_AUTOMATIC
-};
-static gchar *GbPolicySymbols[] =
-{
-  "GTK_POLICY_ALWAYS",
-  "GTK_POLICY_AUTOMATIC"
-};
 
 static void show_ctree_dialog (GbWidgetNewData * data);
-static void on_ctree_dialog_ok (GtkWidget * widget, GbWidgetNewData * data);
-static void on_ctree_dialog_destroy (GtkWidget * widget, GbWidgetNewData * data);
+static void on_ctree_dialog_ok (GtkWidget * widget,
+				GbWidgetNewData * data);
+static void on_ctree_dialog_destroy (GtkWidget * widget,
+				     GbWidgetNewData * data);
 static GtkWidget *new_label (GtkWidget * parent);
+static GtkWidget *new_unnamed_label (GtkWidget * parent);
 
-static void hadjustment_value_changed (GtkAdjustment * adjustment,
-				       GtkWidget *ctree);
-static void vadjustment_value_changed (GtkAdjustment * adjustment,
-				       GtkWidget *ctree);
 
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
@@ -123,8 +109,7 @@ GtkWidget *
 gb_ctree_new (GbWidgetNewData * data)
 {
   GtkWidget *new_widget;
-  GtkAdjustment *adjustment;
-  gint cols = 0;
+  gint cols = 0, i;
 
   if (data->action == GB_LOADING)
     {
@@ -134,15 +119,14 @@ gb_ctree_new (GbWidgetNewData * data)
       /* FIXME: Allow setting of tree column - 2nd arg */
       new_widget = gtk_ctree_new (cols, 0);
 
-      /* Connect signals for redrawing. */
-      adjustment = gtk_range_get_adjustment (GTK_RANGE (GTK_CLIST (new_widget)->hscrollbar));
-      gtk_signal_connect_after (GTK_OBJECT (adjustment), "value_changed",
-				(GtkSignalFunc) hadjustment_value_changed,
-				new_widget);
-      adjustment = gtk_range_get_adjustment (GTK_RANGE (GTK_CLIST (new_widget)->vscrollbar));
-      gtk_signal_connect_after (GTK_OBJECT (adjustment), "value_changed",
-				(GtkSignalFunc) vadjustment_value_changed,
-				new_widget);
+      /* GtkCList has problems if the title buttons aren't created. */
+      for (i = 0; i < cols; i++)
+	{
+	  gtk_clist_set_column_widget (GTK_CLIST (new_widget), i,
+				       new_unnamed_label (new_widget));
+	  gtk_clist_set_column_width (GTK_CLIST (new_widget), i, 80);
+          editor_add_mouse_signals_to_existing (GTK_CLIST (new_widget)->column[i].button);
+	}
 
       return new_widget;
     }
@@ -157,29 +141,17 @@ gb_ctree_new (GbWidgetNewData * data)
 static void
 show_ctree_dialog (GbWidgetNewData * data)
 {
-  GtkWidget *window, *vbox, *hbox, *separator, *hbbox, *label, *spinbutton;
-  GtkWidget *button;
+  GtkWidget *dialog, *vbox, *hbox, *label, *spinbutton;
   GtkObject *adjustment;
 
-  window = gtk_window_new (GTK_WINDOW_DIALOG);
-  gtk_window_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
-  gtk_window_set_title (GTK_WINDOW (window), _("New columned tree"));
-
-  gtk_signal_connect_object (GTK_OBJECT (window), "delete_event",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (window));
-  gtk_signal_connect (GTK_OBJECT (window), "destroy",
-		      GTK_SIGNAL_FUNC (on_ctree_dialog_destroy),
-		      data);
-
-  vbox = gtk_vbox_new (FALSE, 5);
-  gtk_container_add (GTK_CONTAINER (window), vbox);
-  /*gtk_container_border_width (GTK_CONTAINER (vbox), 10); */
-  gtk_widget_show (vbox);
+  dialog = glade_util_create_dialog (_("New columned tree"), data->parent,
+				     on_ctree_dialog_ok, data, &vbox);
+  gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
+		      GTK_SIGNAL_FUNC (on_ctree_dialog_destroy), data);
 
   hbox = gtk_hbox_new (FALSE, 5);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 5);
-  gtk_container_border_width (GTK_CONTAINER (hbox), 10);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox), 10);
   gtk_widget_show (hbox);
 
   label = gtk_label_new (_("Number of columns:"));
@@ -189,56 +161,30 @@ show_ctree_dialog (GbWidgetNewData * data)
   adjustment = gtk_adjustment_new (3, 1, 100, 1, 10, 10);
   spinbutton = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
   /* save pointer to spinbutton so we can find it easily in the OK handler */
-  gtk_object_set_data (GTK_OBJECT (window), "spinbutton", spinbutton);
+  gtk_object_set_data (GTK_OBJECT (dialog), "cols", spinbutton);
   gtk_box_pack_start (GTK_BOX (hbox), spinbutton, TRUE, TRUE, 5);
   gtk_widget_set_usize (spinbutton, 50, -1);
   gtk_widget_grab_focus (spinbutton);
   gtk_widget_show (spinbutton);
 
-  separator = gtk_hseparator_new ();
-  gtk_box_pack_start (GTK_BOX (vbox), separator, TRUE, TRUE, 5);
-  gtk_widget_show (separator);
-
-  hbbox = gtk_hbutton_box_new ();
-  gtk_box_pack_start (GTK_BOX (vbox), hbbox, TRUE, TRUE, 5);
-  gtk_container_border_width (GTK_CONTAINER (vbox), 10);
-  gtk_widget_show (hbbox);
-
-  button = gtk_button_new_with_label (_("OK"));
-  gtk_container_add (GTK_CONTAINER (hbbox), button);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_grab_default (button);
-  gtk_widget_show (button);
-  gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      GTK_SIGNAL_FUNC (on_ctree_dialog_ok),
-		      data);
-
-  button = gtk_button_new_with_label (_("Cancel"));
-  gtk_container_add (GTK_CONTAINER (hbbox), button);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_show (button);
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (window));
-
-  gtk_widget_show (window);
-  gtk_grab_add (window);
+  gtk_widget_show (dialog);
+  gtk_grab_add (dialog);
 }
 
 
 static void
-on_ctree_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
+on_ctree_dialog_ok (GtkWidget * widget,
+		    GbWidgetNewData * data)
 {
-  GtkWidget *new_widget, *spinbutton, *window;
-  GtkAdjustment *adjustment;
+  GtkWidget *new_widget, *spinbutton, *dialog, *label;
   gint cols, i;
 
-  window = gtk_widget_get_toplevel (widget);
+  dialog = gtk_widget_get_toplevel (widget);
 
   /* Only call callback if placeholder/fixed widget is still there */
   if (gb_widget_can_finish_new (data))
     {
-      spinbutton = gtk_object_get_data (GTK_OBJECT (window), "spinbutton");
+      spinbutton = gtk_object_get_data (GTK_OBJECT (dialog), "cols");
       g_return_if_fail (spinbutton != NULL);
       cols = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spinbutton));
 
@@ -247,30 +193,22 @@ on_ctree_dialog_ok (GtkWidget * widget, GbWidgetNewData * data)
       gtk_clist_column_titles_show (GTK_CLIST (new_widget));
       for (i = 0; i < cols; i++)
 	{
+	  label = new_label (new_widget);
 	  gtk_clist_set_column_widget (GTK_CLIST (new_widget), i,
-				       new_label (new_widget));
+				       label);
 	  gtk_clist_set_column_width (GTK_CLIST (new_widget), i, 80);
 	}
 
-      /* Connect signals for redrawing. */
-      adjustment = gtk_range_get_adjustment (GTK_RANGE (GTK_CLIST (new_widget)->hscrollbar));
-      gtk_signal_connect_after (GTK_OBJECT (adjustment), "value_changed",
-				(GtkSignalFunc) hadjustment_value_changed,
-				new_widget);
-      adjustment = gtk_range_get_adjustment (GTK_RANGE (GTK_CLIST (new_widget)->vscrollbar));
-      gtk_signal_connect_after (GTK_OBJECT (adjustment), "value_changed",
-				(GtkSignalFunc) vadjustment_value_changed,
-				new_widget);
-
       gb_widget_initialize (new_widget, data);
       (*data->callback) (new_widget, data);
     }
-  gtk_widget_destroy (window);
+  gtk_widget_destroy (dialog);
 }
 
 
 static void
-on_ctree_dialog_destroy (GtkWidget * widget, GbWidgetNewData * data)
+on_ctree_dialog_destroy (GtkWidget * widget,
+			 GbWidgetNewData * data)
 {
   gb_widget_free_new_data (data);
   gtk_grab_remove (widget);
@@ -284,57 +222,24 @@ new_label (GtkWidget * parent)
 
   label = gb_widget_new ("GtkLabel", parent);
   g_return_val_if_fail (label != NULL, NULL);
-  gtk_object_set_data (GTK_OBJECT (label), GB_CHILD_NAME_KEY,
-		       g_strdup (CTreeTitle));
+  gb_widget_set_child_name (label, CTreeTitle);
   return label;
 }
 
 
-/* These two functions redraw the ctree when the scrollbars are used, so
-   that the selection rectangle doesn't get messed up - the ctree uses
-   gdk_window_copy_area() which will copy the selection handles which we
-   do not want to happen. */
-static void
-hadjustment_value_changed (GtkAdjustment * adjustment,
-			   GtkWidget *ctree)
+GtkWidget *
+new_unnamed_label (GtkWidget * parent)
 {
-  gint x, y, w, h;
+  GtkWidget *label;
 
-  x = ctree->allocation.x;
-  y = ctree->allocation.y;
-  w = ctree->allocation.width;
-  h = ctree->allocation.height;
-
-  editor_refresh_widget_area (ctree, x, y, w, GB_CORNER_HEIGHT);
-  editor_refresh_widget_area (ctree, x, y + h - GB_CORNER_HEIGHT,
-			      w, GB_CORNER_HEIGHT);
-  gtk_widget_draw (ctree, NULL);
-  /* FIXME: GTK bug workaround to draw CTree children. */
-  gtk_widget_draw_children (ctree);
+  label = gb_widget_new_full ("GtkLabel", FALSE, parent, NULL, 0, 0, NULL,
+			      GB_CREATING, NULL);
+  g_return_val_if_fail (label != NULL, NULL);
+  gb_widget_set_child_name (label, CTreeTitle);
+  return label;
 }
 
 
-static void
-vadjustment_value_changed (GtkAdjustment * adjustment,
-			   GtkWidget *ctree)
-{
-  gint x, y, w, h;
-
-  x = ctree->allocation.x;
-  y = ctree->allocation.y;
-  w = ctree->allocation.width;
-  h = ctree->allocation.height;
-
-  editor_refresh_widget_area (ctree, x, y, GB_CORNER_WIDTH, h);
-  editor_refresh_widget_area (ctree, x + w - GB_CORNER_WIDTH, y,
-			      GB_CORNER_WIDTH, h);
-  gtk_widget_draw (ctree, NULL);
-  /* FIXME: GTK bug workaround to draw CTree children. */
-  gtk_widget_draw_children (ctree);
-}
-
-
-
 /*
  * Creates the components needed to edit the extra properties of this widget.
  */
@@ -349,12 +254,6 @@ gb_ctree_create_properties (GtkWidget * widget, GbWidgetCreateArgData * data)
   property_add_choice (Shadow, _("Shadow:"),
 		       _("The type of shadow of the columned tree's border"),
 		       GbShadowChoices);
-  property_add_choice (HPolicy, _("H Policy:"),
-		       _("When the horizontal scrollbar will be shown"),
-		       GbPolicyChoices);
-  property_add_choice (VPolicy, _("V Policy:"),
-		       _("When the vertical scrollbar will be shown"),
-		       GbPolicyChoices);
 }
 
 
@@ -409,17 +308,6 @@ gb_ctree_get_properties (GtkWidget *widget, GbWidgetGetArgData * data)
       if (GbShadowValues[i] == GTK_CLIST (widget)->shadow_type)
 	gb_widget_output_choice (data, Shadow, i, GbShadowSymbols[i]);
     }
-
-  for (i = 0; i < sizeof (GbPolicyValues) / sizeof (GbPolicyValues[0]); i++)
-    {
-      if (GbPolicyValues[i] == GTK_CLIST (widget)->hscrollbar_policy)
-	gb_widget_output_choice (data, HPolicy, i, GbPolicySymbols[i]);
-    }
-  for (i = 0; i < sizeof (GbPolicyValues) / sizeof (GbPolicyValues[0]); i++)
-    {
-      if (GbPolicyValues[i] == GTK_CLIST (widget)->vscrollbar_policy)
-	gb_widget_output_choice (data, VPolicy, i, GbPolicySymbols[i]);
-    }
 }
 
 
@@ -432,11 +320,9 @@ static void
 gb_ctree_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 {
   gchar *widths, *pos, *mode;
-  gchar *shadow, *hpolicy, *vpolicy;
-  gboolean myApply, titles;
+  gchar *shadow;
+  gboolean titles;
   gint col, w, i;
-  GtkPolicyType hpolicy_value = GTK_POLICY_AUTOMATIC;
-  GtkPolicyType vpolicy_value = GTK_POLICY_AUTOMATIC;
 
   if (data->action == GB_LOADING)
     {
@@ -488,29 +374,12 @@ gb_ctree_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 	  if (!strcmp (shadow, GbShadowChoices[i])
 	      || !strcmp (shadow, GbShadowSymbols[i]))
 	    {
-	      gtk_clist_set_border (GTK_CLIST (widget), GbShadowValues[i]);
+	      gtk_clist_set_shadow_type (GTK_CLIST (widget),
+					 GbShadowValues[i]);
 	      break;
 	    }
 	}
     }
-
-  hpolicy = gb_widget_input_choice (data, HPolicy);
-  myApply = data->apply;
-  vpolicy = gb_widget_input_choice (data, VPolicy);
-  if (data->apply || myApply)
-    {
-      for (i = 0; i < sizeof (GbPolicyValues) / sizeof (GbPolicyValues[0]); i
-	   ++)
-	{
-	  if (!strcmp (hpolicy, GbPolicyChoices[i])
-	      || !strcmp (hpolicy, GbPolicySymbols[i]))
-	    hpolicy_value = GbPolicyValues[i];
-	  if (!strcmp (vpolicy, GbPolicyChoices[i])
-	      || !strcmp (vpolicy, GbPolicySymbols[i]))
-	    vpolicy_value = GbPolicyValues[i];
-	}
-      gtk_clist_set_policy (GTK_CLIST (widget), vpolicy_value, hpolicy_value);
-    }
 }
 
 
@@ -529,6 +398,36 @@ gb_ctree_create_popup_menu (GtkWidget * widget, GbWidgetCreateMenuData * data)
 */
 
 
+void
+gb_ctree_add_child (GtkWidget *widget, GtkWidget *child,
+		    GbWidgetSetArgData *data)
+{
+  gchar *child_name = load_get_value (data, "child_name");
+
+  if (child_name && (!strcmp (child_name, CTreeTitle)))
+    {
+      /* We store the last column title read in 'last_child' */
+      gint col = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget),
+                                                       "last_child"));
+
+      gtk_clist_set_column_widget (GTK_CLIST (widget), col, child);
+      gtk_object_set_data (GTK_OBJECT (widget), "last_child",
+                           GINT_TO_POINTER (col + 1));
+
+      /* We need to add signals to the clist button, just in case the
+         title widget has no window and so doesn't get signals itself.
+         Since Clist always creates 1 button initially, the signals would
+         be added to this button in gb_widget_new, so we could skip it,
+         but it doesn't hurt. */
+      editor_add_mouse_signals_to_existing (GTK_CLIST (widget)->column[col].button);
+    }
+  else
+    {
+      g_warning ("Adding unknown child to a GtkCTree");
+      gtk_container_add (GTK_CONTAINER (widget), child);
+    }
+}
+
 
 /*
  * Writes the source code needed to create this widget.
@@ -538,7 +437,7 @@ gb_ctree_create_popup_menu (GtkWidget * widget, GbWidgetCreateMenuData * data)
 static void
 gb_ctree_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 {
-  gint col, i, hpolicy = 0, vpolicy = 0;
+  gint col, i;
 
   if (data->create_widget)
     {
@@ -584,26 +483,42 @@ gb_ctree_write_source (GtkWidget * widget, GbWidgetWriteSourceData * data)
 	   ++)
 	{
 	  if (GbShadowValues[i] == GTK_CLIST (widget)->shadow_type)
-	    source_add (data, "  gtk_clist_set_border (GTK_CLIST (%s), %s);\n",
+	    source_add (data, "  gtk_clist_set_shadow_type (GTK_CLIST (%s), %s);\n",
 			data->wname, GbShadowSymbols[i]);
 	}
     }
+}
 
-  if (GTK_CLIST (widget)->hscrollbar_policy != GTK_POLICY_ALWAYS
-      || GTK_CLIST (widget)->vscrollbar_policy != GTK_POLICY_ALWAYS)
+
+/* Outputs source to add a child widget to a CTree. */
+static void
+gb_ctree_write_add_child_source (GtkWidget * parent,
+				 const gchar *parent_name,
+				 GtkWidget *child,
+				 GbWidgetWriteSourceData * data)
+{
+  gchar *child_name;
+
+  child_name = gb_widget_get_child_name (child);
+
+  /* See if this is a title widget. */
+  if (child_name && (!strcmp (child_name, CTreeTitle)))
     {
-      for (i = 0; i < sizeof (GbPolicyValues) / sizeof (GbPolicyValues[0]); i
-	   ++)
-	{
-	  if (GbPolicyValues[i] == GTK_CLIST (widget)->hscrollbar_policy)
-	    hpolicy = i;
-	  if (GbPolicyValues[i] == GTK_CLIST (widget)->vscrollbar_policy)
-	    vpolicy = i;
-	}
+      /* We store the last column title written in 'last_child' */
+      gint col = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (parent),
+						       "last_child"));
+      source_add (data,
+		  "  gtk_clist_set_column_widget (GTK_CLIST (%s), %i, %s);\n",
+		  parent_name, col + 1, data->wname);
 
-      source_add (data, "  gtk_clist_set_policy (GTK_CLIST (%s), %s, %s);\n",
-		  data->wname, GbPolicySymbols[vpolicy],
-		  GbPolicySymbols[hpolicy]);
+      gtk_object_set_data (GTK_OBJECT (parent), "last_child",
+			   GINT_TO_POINTER (col + 1));
+    }
+  else
+    {
+      g_warning ("Unknown CTree child widgetL %s", data->wname);
+      source_add (data, "  gtk_container_add (GTK_CONTAINER (%s), %s);\n",
+		  parent_name, data->wname);
     }
 }
 
@@ -629,10 +544,12 @@ gb_ctree_init ()
 
   /* Fill in any functions that this GbWidget has */
   gbwidget.gb_widget_new		= gb_ctree_new;
+  gbwidget.gb_widget_add_child		= gb_ctree_add_child;
   gbwidget.gb_widget_create_properties	= gb_ctree_create_properties;
   gbwidget.gb_widget_get_properties	= gb_ctree_get_properties;
   gbwidget.gb_widget_set_properties	= gb_ctree_set_properties;
   gbwidget.gb_widget_write_source	= gb_ctree_write_source;
+  gbwidget.gb_widget_write_add_child_source = gb_ctree_write_add_child_source;
 /*
   gbwidget.gb_widget_create_popup_menu	= gb_ctree_create_popup_menu;
 */
@@ -640,4 +557,3 @@ gb_ctree_init ()
   return &gbwidget;
 }
 
-#endif /* GLD_HAVE_GTK_1_1 */
