@@ -130,11 +130,16 @@ gb_notebook_add_child (GtkWidget *widget, GtkWidget *child,
       if (elem)
 	{
 	  notebook_page = ((GtkNotebookPage*) elem->data)->child;
+#if 1
+	  gtk_notebook_set_tab_label (GTK_NOTEBOOK (widget), notebook_page,
+				      child);
+#else
 	  gtk_widget_ref (notebook_page);
 	  gtk_notebook_remove_page (GTK_NOTEBOOK (widget), pos);
 	  gtk_notebook_insert_page (GTK_NOTEBOOK (widget), notebook_page,
 				    child, pos);
 	  gtk_widget_unref (notebook_page);
+#endif
 	}
       else
 	{
