@@ -97,9 +97,10 @@ const gchar *GladeOrientationSymbols[] =
  * Stock Gnome Menu Items.
  *************************************************************************/
 
-/* This is the index of the new file item, so if you change the
+/* These are indices of a few special items, so if you change the
    GladeStockMenuItemValues array be sure to update this. */
 const gint GladeStockMenuItemNew = 3;
+const gint GladeStockMenuHelpTree = 37;
 
 /* These are copied from gnome-libs/libgnomeui/gnome-app-helper.h
    The first item is 'None' to indicate no stock item is being used. */
@@ -883,6 +884,8 @@ glade_gnome_start_menu_source (GtkMenuShell * menu_shell,
 			       GbWidgetWriteSourceData * data)
 {
   GString *source_buffer;
+  GtkWidget *parent_item;
+  gint uiinfo_index = 0, stock_item_index;
 
   source_buffer = g_string_sized_new (1024);
   g_string_sprintf (source_buffer, "static GnomeUIInfo %s_uiinfo[] =\n{\n",
@@ -890,8 +893,29 @@ glade_gnome_start_menu_source (GtkMenuShell * menu_shell,
   gtk_object_set_data (GTK_OBJECT (menu_shell), "glade-uiinfo", source_buffer);
   gtk_object_set_data (GTK_OBJECT (menu_shell), "glade-uiinfo-name",
 		       g_strdup (data->real_wname));
+
+  /* See if this is the start of the 'Help' menu, and if it is output the
+     GNOMEUIINFO_HELP item which inserts all the topics from topic.dat. */
+  if (glade_project_get_gnome_help_support (data->project)
+      && GTK_IS_MENU (menu_shell))
+    {
+      parent_item = gtk_menu_get_attach_widget (GTK_MENU (menu_shell));
+
+      if (parent_item)
+	{
+	  stock_item_index = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (parent_item), GladeMenuItemStockIndexKey));
+	  if (stock_item_index == GladeStockMenuHelpTree)
+	    {
+	      g_string_sprintfa (source_buffer, "  GNOMEUIINFO_HELP (%s),\n",
+				 source_make_string (data->program_name,
+						     FALSE));
+	      uiinfo_index++;
+	    }
+	}
+    }
+
   gtk_object_set_data (GTK_OBJECT (menu_shell), "glade-uiinfo-index",
-		       GINT_TO_POINTER (0));
+		       GINT_TO_POINTER (uiinfo_index));
 }
 
 
@@ -1200,6 +1224,7 @@ static void
 glade_gnome_write_menu_item_standard_source (GtkMenuItem * widget,
 					     GbWidgetWriteSourceData * data)
 {
+  GbWidgetData *wdata = data->widget_data;
   gchar *uiinfo_name, *uiinfo_name_key, *uiinfo_index_key;
   gint uiinfo_index;
 
@@ -1238,6 +1263,16 @@ glade_gnome_write_menu_item_standard_source (GtkMenuItem * widget,
 		  uiinfo_name, uiinfo_index);
     }
 
+  if (GTK_IS_CHECK_MENU_ITEM (widget) && wdata->flags & GB_ACTIVE)
+    source_add (data,
+		"  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (%s_uiinfo[%i].widget), TRUE);\n",
+		uiinfo_name, uiinfo_index);
+
+  if (!(wdata->flags & GB_SENSITIVE))
+    source_add (data,
+		"  gtk_widget_set_sensitive (%s_uiinfo[%i].widget, FALSE);\n",
+		uiinfo_name, uiinfo_index);
+
   gtk_object_set_data (GTK_OBJECT (GTK_WIDGET (widget)->parent),
 		       uiinfo_index_key, GINT_TO_POINTER (uiinfo_index + 1));
 }
@@ -1421,16 +1456,18 @@ glade_gnome_is_gnome_dialog_button (GtkWidget *widget)
 }
 
 
-gboolean
+/* If the given widget is a dock item inside a GnomeApp it returns the
+   GnomeApp, otherwise it returns NULL. */
+GnomeApp*
 glade_gnome_is_app_dock_item (GtkWidget *widget)
 {
-  GtkWidget *dock;
+  GtkWidget *dock, *app;
 
   if (!GNOME_IS_DOCK_ITEM (widget))
-    return FALSE;
+    return NULL;
 
   if (!widget->parent)
-    return FALSE;
+    return NULL;
 
   /* Floating items are direct children of a GnomeDock.
      Other items are children of a GnomeDockBand. */
@@ -1441,19 +1478,20 @@ glade_gnome_is_app_dock_item (GtkWidget *widget)
   else
     {
       if (!GNOME_IS_DOCK_BAND (widget->parent))
-	return FALSE;
+	return NULL;
       dock = widget->parent->parent;
       if (!dock || !GNOME_IS_DOCK (dock))
-	return FALSE;
+	return NULL;
     }
 
   if (!dock->parent || !GTK_IS_VBOX (dock->parent))
-    return FALSE;
+    return NULL;
 
-  if (!dock->parent->parent || !GNOME_IS_APP (dock->parent->parent))
-    return FALSE;
+  app = dock->parent->parent;
+  if (!app || !GNOME_IS_APP (app))
+    return NULL;
 
-  return TRUE;
+  return GNOME_APP (app);
 }
 
 
