@@ -178,6 +178,9 @@ glade_project_init (GladeProject * project)
   project->gettext_support = FALSE;
 #endif
 
+  project->gnome_db_support = FALSE;
+  project->gnome_help_support = FALSE;
+
   project->use_widget_names = FALSE;
   project->output_main_file = TRUE;
   project->output_support_files = TRUE;
@@ -924,6 +927,25 @@ glade_project_set_gnome_support	(GladeProject  *project,
 }
 
 
+gboolean
+glade_project_get_gnome_db_support	(GladeProject  *project)
+{
+  return project->gnome_db_support;
+}
+
+
+void
+glade_project_set_gnome_db_support	(GladeProject  *project,
+					 gboolean       gnome_db_support)
+{
+  if (project->gnome_db_support == gnome_db_support)
+    return;
+
+  project->gnome_db_support = gnome_db_support;
+  glade_project_set_changed (project, TRUE);
+}
+
+
 /*
  * C Output options.
  */
@@ -1035,6 +1057,25 @@ glade_project_set_backup_source_files	(GladeProject  *project,
 }
 
 
+gboolean
+glade_project_get_gnome_help_support	(GladeProject  *project)
+{
+  return project->gnome_help_support;
+}
+
+
+void
+glade_project_set_gnome_help_support	(GladeProject  *project,
+					 gboolean       gnome_help_support)
+{
+  if (project->gnome_help_support == gnome_help_support)
+    return;
+
+  project->gnome_help_support = gnome_help_support;
+  glade_project_set_changed (project, TRUE);
+}
+
+
 /* FIXME: These will be removed when the source code output is improved. */
 void
 glade_project_get_source_files (GladeProject *project,
@@ -1239,6 +1280,18 @@ glade_project_load_options (GladeProject *project,
 #endif
 	  project->gnome_support = gnome_support;
 	}
+      else if (!strcmp (element, "gnome_db_support"))
+	{
+	  gboolean gnome_db_support = load_parse_bool (data, cdata);
+#ifndef	USE_GNOME_DB
+	  if (gnome_db_support == TRUE)
+	    {
+	      g_warning ("Glade has been compiled without support for Gnome DB.");
+	      data->status = GLADE_STATUS_ERROR;
+	    }
+#endif
+	  project->gnome_db_support = gnome_db_support;
+	}
       else if ((!strcmp (element, "gettext_support"))
 	       || (!strcmp (element, "i18n_strings"))) /* Old name. */
 	{
@@ -1264,7 +1317,10 @@ glade_project_load_options (GladeProject *project,
 	{
 	  project->backup_source_files = load_parse_bool (data, cdata);
 	}
-
+      else if (!strcmp (element, "gnome_help_support"))
+	{
+	  project->gnome_help_support = load_parse_bool (data, cdata);
+	}
       else if (!strcmp (element, "main_source_file"))
 	{
 	  g_free (project->main_source_file);
@@ -1365,6 +1421,10 @@ glade_project_save_options (GladeProject *project,
   /*if (!project->gnome_support)*/
     save_bool   (data, "gnome_support", project->gnome_support);
 
+  if (project->gnome_db_support)
+    save_bool   (data, "gnome_db_support", project->gnome_db_support);
+
+
   /*
    * C Options.
    */
@@ -1380,6 +1440,8 @@ glade_project_save_options (GladeProject *project,
     save_bool   (data, "output_build_files", project->output_build_files);
   if (!project->backup_source_files)
     save_bool   (data, "backup_source_files", project->backup_source_files);
+  if (project->gnome_help_support)
+    save_bool   (data, "gnome_help_support", project->gnome_help_support);
 
   if (!project->main_source_file
       || strcmp (project->main_source_file, "interface.c"))
@@ -1616,7 +1678,7 @@ glade_project_real_ensure_widgets_named (GtkWidget    *widget,
 {
   if (GB_IS_GB_WIDGET (widget) && widget->name == NULL)
     {
-      gchar *class = gb_widget_get_type_name (widget);
+      gchar *class = gb_widget_get_class_id (widget);
       gtk_widget_set_name (widget,
 			   glade_project_new_widget_name (project, class));
       MSG1 ("set default name for widget: %s\n", gtk_widget_get_name (widget));
