@@ -399,12 +399,25 @@ glade_project_options_init (GladeProjectOptions * options)
   gtk_widget_show (frame);
   gtk_box_pack_start (GTK_BOX (general_options_vbox), frame, FALSE, TRUE, 4);
 
+  hbox = gtk_hbox_new (TRUE, 4);
+  gtk_widget_show (hbox);
+  gtk_container_add (GTK_CONTAINER (frame), hbox);
+
   options->gnome_support = gtk_check_button_new_with_label (_("Enable Gnome Support"));
   gtk_widget_show (options->gnome_support);
-  gtk_container_add (GTK_CONTAINER (frame), options->gnome_support);
+  gtk_box_pack_start (GTK_BOX (hbox), options->gnome_support, TRUE, TRUE, 0);
   gtk_tooltips_set_tip (tooltips, options->gnome_support,
 			_("If a Gnome application is to be built"), NULL);
-#endif
+
+#ifdef USE_GNOME_DB
+  options->gnome_db_support = gtk_check_button_new_with_label (_("Enable Gnome DB Support"));
+  gtk_widget_show (options->gnome_db_support);
+  gtk_box_pack_start (GTK_BOX (hbox), options->gnome_db_support, TRUE, TRUE, 0);
+  gtk_tooltips_set_tip (tooltips, options->gnome_db_support,
+			_("If a Gnome DB application is to be built"), NULL);
+#endif /* USE_GNOME_DB */
+
+#endif /* USE_GNOME */
 
 
   /*
@@ -460,6 +473,17 @@ glade_project_options_init (GladeProjectOptions * options)
 			_("If copies of old source files are made"),
 			NULL);
 
+  /* Gnome Help System support. */
+  options->gnome_help_support = gtk_check_button_new_with_label (_("Gnome Help Support"));
+#ifdef USE_GNOME
+  gtk_widget_show (options->gnome_help_support);
+#endif
+  gtk_box_pack_start (GTK_BOX (vbox), options->gnome_help_support,
+		      FALSE, FALSE, 0);
+  gtk_tooltips_set_tip (tooltips, options->gnome_help_support,
+			_("If support for the Gnome Help system should be included"),
+			NULL);
+
   frame = gtk_frame_new (_("File Output Options:"));
   gtk_widget_show (frame);
   gtk_box_pack_start (GTK_BOX (hbox), frame, FALSE, TRUE, 0);
@@ -886,6 +910,10 @@ glade_project_options_set_project (GladeProjectOptions *options,
 #ifdef USE_GNOME
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (options->gnome_support),
 				project->gnome_support);
+#ifdef USE_GNOME_DB
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (options->gnome_db_support),
+				project->gnome_db_support);
+#endif
 #endif
 
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (options->gettext_support),
@@ -900,6 +928,8 @@ glade_project_options_set_project (GladeProjectOptions *options,
 				project->output_build_files);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (options->backup_source_files),
 				project->backup_source_files);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (options->gnome_help_support),
+				project->gnome_help_support);
 
   gtk_entry_set_text (GTK_ENTRY (options->main_source_entry),
 		      project->main_source_file
@@ -1218,6 +1248,9 @@ glade_project_options_ok (GtkWidget *widget,
 
 #ifdef USE_GNOME
   glade_project_set_gnome_support (project, GTK_TOGGLE_BUTTON (options->gnome_support)->active ? TRUE : FALSE);
+#ifdef USE_GNOME_DB
+  glade_project_set_gnome_db_support (project, GTK_TOGGLE_BUTTON (options->gnome_db_support)->active ? TRUE : FALSE);
+#endif
 #endif
 
   glade_project_set_gettext_support (project, GTK_TOGGLE_BUTTON (options->gettext_support)->active ? TRUE : FALSE);
@@ -1232,6 +1265,8 @@ glade_project_options_ok (GtkWidget *widget,
 
   glade_project_set_backup_source_files (project, GTK_TOGGLE_BUTTON (options->backup_source_files)->active ? TRUE : FALSE);
 
+  glade_project_set_gnome_help_support (project, GTK_TOGGLE_BUTTON (options->gnome_help_support)->active ? TRUE : FALSE);
+
   glade_project_set_source_files (project,
 				  get_entry_text (options->main_source_entry),
 				  get_entry_text (options->main_header_entry),
