@@ -272,8 +272,10 @@ static GnomeUIInfo SettingsMenu[] =
 
 static GnomeUIInfo HelpMenu[] =
 {
-  /* Currently we don't have any help topics. */
-  /*GNOMEUIINFO_HELP ("Glade"),*/
+  GNOMEUIINFO_HELP ("glade"),
+
+  GNOMEUIINFO_SEPARATOR,
+
   GNOMEUIINFO_MENU_ABOUT_ITEM (glade_project_window_about, NULL),
   GNOMEUIINFO_END
 };
@@ -1544,12 +1546,11 @@ glade_project_window_about (GtkWidget *widget,
 
   /* VERSION comes from configure.in - the only place it should be defined */
 #ifdef USE_GNOME
-  about = gnome_about_new ("Glade",
+  about = gnome_about_new (_("Glade"),
 			   VERSION,
-			   "Copyright 1998 Damon Chaplin",
+			   _("Copyright 1998 Damon Chaplin"),
 			   author,
-			   "A GTK+ User Interface Builder\n"
-			   "Web: http://glade.pn.org\n",
+			   _("A GTK+ User Interface Builder\nWeb: http://glade.pn.org\n"),
 			   PACKAGE G_DIR_SEPARATOR_S "glade_logo.png");
   gtk_window_set_modal (GTK_WINDOW (about), TRUE);
   gtk_window_set_wmclass (GTK_WINDOW (about), "about", "Glade");
