@@ -300,6 +300,12 @@ void	    property_set_font		(const gchar	    *property_name,
 					 GdkFont	    *font,
 					 const gchar	    *xlfd_fontname);
 
+void	    property_add_command	(const gchar	    *property_name,
+					 const gchar	    *label,
+					 const gchar	    *tooltip,
+					 const gchar	    *command,
+					 GtkSignalFunc	     callback);
+
 #define GB_ADJUST_DEFAULT_LABELS 1
 #define GB_ADJUST_H_LABELS	 2
 #define GB_ADJUST_V_LABELS	 3
