@@ -645,6 +645,11 @@ source_write_interface_c_preamble (GbWidgetWriteSourceData * data, FILE *fp)
   if (glade_project_get_gnome_support (data->project))
     {
       fprintf (fp, "#include <gnome.h>\n");
+
+      if (glade_project_get_gnome_db_support (data->project))
+	{
+	  fprintf (fp, "#include <gda-ui.h>\n");
+	}
     }
   else
     {
@@ -1416,6 +1421,21 @@ source_write_gnome_configure_in (GbWidgetWriteSourceData * data)
 	   "AC_SUBST(PACKAGE_PIXMAPS_DIR)\n"
 	   "\n");
 
+  /* GnomeDB support. */
+  if (glade_project_get_gnome_db_support (data->project))
+    {
+      fprintf (fp,
+	       "dnl Gnome DB support.\n"
+	       "AC_PATH_PROG(gda_config, gda-config)\n"
+	       "if test \"_$gda_config\" != _; then\n"
+	       "  CPPFLAGS=\"`$gda_config --cflags 2>/dev/null` $CPPFLAGS\"\n"
+	       "  LIBS=\"`$gda_config --libs gdaui 2>/dev/null` $LIBS\"\n"
+	       "else\n"
+	       "  AC_MSG_ERROR(gda-config not found. Check you have Gnome DB installed.)\n"
+	       "fi\n"
+	       "\n");
+    }
+
   fprintf (fp,
 	   "AC_OUTPUT([\n"
 	   "Makefile\n"
@@ -1628,6 +1648,7 @@ source_write_gnome_macro_files (GbWidgetWriteSourceData * data)
     "gnome-x-checks.m4",
     "gnome-xml-check.m4",
     "gnome.m4",
+    "gperf-check.m4",
     "linger.m4",
     "need-declaration.m4",
     NULL
@@ -2245,6 +2266,12 @@ source_write_gtk_create_pixmap_functions (GbWidgetWriteSourceData * data,
 	   "  GtkWidget *pixmap;\n"
 	   "  GList *elem;\n"
 	   "\n");
+
+  fprintf (fp,
+	   "  if (!filename || !filename[0])\n"
+	   "      return create_dummy_pixmap (widget);\n"
+	   "\n");
+
   fprintf (fp,
 	   "  /* We first try any pixmaps directories set by the application. */\n"
 	   "  elem = pixmaps_directories;\n"
@@ -2398,6 +2425,11 @@ source_write_gnome_create_pixmap_functions (GbWidgetWriteSourceData * data,
 	   "  gchar *pathname;\n"
 	   "\n");
 
+  fprintf (fp,
+	   "  if (!filename || !filename[0])\n"
+	   "      return create_dummy_pixmap (widget, gnome_pixmap);\n"
+	   "\n");
+
   fprintf (fp,
 	   "  pathname = gnome_pixmap_file (filename);\n"
 	   "  if (!pathname)\n"
@@ -2708,7 +2740,10 @@ source_create_pixmap (GbWidgetWriteSourceData * data,
 		      const gchar             * filename,
 		      gboolean			gnome_pixmap)
 {
-  g_return_if_fail (filename != NULL && filename[0] != '\0');
+  gboolean empty_filename = FALSE;
+
+  if (!filename || filename[0] == '\0')
+    empty_filename = TRUE;
 
   /* We use the basename of the pixmap file. The create_pixmap() support
      function is responsible for finding the pixmap.
@@ -2717,15 +2752,24 @@ source_create_pixmap (GbWidgetWriteSourceData * data,
   if (glade_project_get_gnome_support (data->project))
     {
       /* FIXME: Should convert filename to a valid C string? */
-      source_add (data, "  %s = create_pixmap (%s, \"%s/%s\", %s);\n",
-		  identifier, data->component_name,
-		  data->program_name, g_basename (filename),
-		  gnome_pixmap ? "TRUE" : "FALSE");
+      if (!empty_filename)
+	source_add (data, "  %s = create_pixmap (%s, \"%s/%s\", %s);\n",
+		    identifier, data->component_name,
+		    data->program_name, g_basename (filename),
+		    gnome_pixmap ? "TRUE" : "FALSE");
+      else
+	source_add (data, "  %s = create_pixmap (%s, NULL, %s);\n",
+		    identifier, data->component_name,
+		    gnome_pixmap ? "TRUE" : "FALSE");
     }
   else
     {
-      source_add (data, "  %s = create_pixmap (%s, \"%s\");\n",
-		  identifier, data->component_name, g_basename (filename));
+      if (!empty_filename)
+	source_add (data, "  %s = create_pixmap (%s, \"%s\");\n",
+		    identifier, data->component_name, g_basename (filename));
+      else
+	source_add (data, "  %s = create_pixmap (%s, NULL);\n",
+		    identifier, data->component_name);
     }
 }
 
