@@ -184,7 +184,7 @@ glade_util_create_dialog_with_buttons (const gchar * message,
 
   for (i = 0; i < nbuttons; i++)
     {
-      button = gtk_button_new_with_label (buttons[i]);
+      button = gtk_button_new_with_label (_(buttons[i]));
       gtk_container_add (GTK_CONTAINER (bbox), button);
       GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
       if (i == default_button - 1)
