@@ -140,6 +140,10 @@ gchar*	    glade_util_create_modifiers_string (guint8 modifier_flags);
 GtkWidget*  glade_util_create_pixmap_using_colormap (GdkColormap *colormap,
 						     gchar **xpm_data);
 
+/* This returns TRUE if the widget is a toplevel project component,
+   i.e. a window, dialog or popup menu. */
+gboolean    glade_util_is_component	(GtkWidget	   *widget);
+
 /* This returns the toplevel widget containing the given widget. It is similar
    to gtk_widget_get_toplevel() but is also walks up menus. */
 GtkWidget*  glade_util_get_toplevel	(GtkWidget	   *widget);
